@@ -95,7 +95,10 @@ util_multi_close(CurlMultiObject *self)
     if (self->multi_handle != NULL) {
         CURLM *multi_handle = self->multi_handle;
         self->multi_handle = NULL;
+        /* Allow threads because callbacks can be invoked */
+        PYCURL_BEGIN_ALLOW_THREADS
         curl_multi_cleanup(multi_handle);
+        PYCURL_END_ALLOW_THREADS
     }
 }
 
@@ -196,7 +199,7 @@ multi_socket_callback(CURL *easy,
     arglist = Py_BuildValue("(iiOO)", what, s, userp, (PyObject *)socketp);
     if (arglist == NULL)
         goto verbose_error;
-    result = PyEval_CallObject(self->s_cb, arglist);
+    result = PyObject_Call(self->s_cb, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL)
         goto verbose_error;
@@ -240,7 +243,7 @@ multi_timer_callback(CURLM *multi,
     arglist = Py_BuildValue("(i)", timeout_ms);
     if (arglist == NULL)
         goto verbose_error;
-    result = PyEval_CallObject(self->t_cb, arglist);
+    result = PyObject_Call(self->t_cb, arglist, NULL);
     Py_DECREF(arglist);
     if (result == NULL)
         goto verbose_error;
@@ -272,6 +275,9 @@ do_multi_setopt_int(CurlMultiObject *self, int option, PyObject *obj)
     case CURLMOPT_MAX_PIPELINE_LENGTH:
     case CURLMOPT_CONTENT_LENGTH_PENALTY_SIZE:
     case CURLMOPT_CHUNK_LENGTH_PENALTY_SIZE:
+#endif
+#ifdef HAVE_CURL_7_67_0_MULTI_STREAMS
+    case CURLMOPT_MAX_CONCURRENT_STREAMS:
 #endif
         curl_multi_setopt(self->multi_handle, option, d);
         break;
@@ -625,7 +631,10 @@ do_multi_add_handle(CurlMultiObject *self, PyObject *args)
     PyDict_SetItem(self->easy_object_dict, (PyObject *) obj, Py_True);
     
     assert(obj->multi_stack == NULL);
+    /* Allow threads because callbacks can be invoked */
+    PYCURL_BEGIN_ALLOW_THREADS
     res = curl_multi_add_handle(self->multi_handle, obj->handle);
+    PYCURL_END_ALLOW_THREADS
     if (res != CURLM_OK) {
         PyDict_DelItem(self->easy_object_dict, (PyObject *) obj);
         CURLERROR_MSG("curl_multi_add_handle() failed due to internal errors");
@@ -660,7 +669,10 @@ do_multi_remove_handle(CurlMultiObject *self, PyObject *args)
         PyErr_SetString(ErrorObject, "curl object not on this multi-stack");
         return NULL;
     }
+    /* Allow threads because callbacks can be invoked */
+    PYCURL_BEGIN_ALLOW_THREADS
     res = curl_multi_remove_handle(self->multi_handle, obj->handle);
+    PYCURL_END_ALLOW_THREADS
     if (res == CURLM_OK) {
         PyDict_DelItem(self->easy_object_dict, (PyObject *) obj);
         // if PyDict_DelItem fails, remove_handle call will also fail.
@@ -797,9 +809,20 @@ do_multi_info_read(CurlMultiObject *self, PyObject *args)
         }
         else {
             /* Create a result tuple that will get added to err_list. */
-            PyObject *v = Py_BuildValue("(Ois)", (PyObject *)co, (int)msg->data.result, co->error);
+            PyObject *error_str = NULL;
+            PyObject *v;
+#if PY_MAJOR_VERSION >= 3
+            error_str = PyUnicode_DecodeLocale(co->error, "surrogateescape");
+            if (error_str == NULL) {
+                goto error;
+            }
+            v = Py_BuildValue("(OiO)", (PyObject *)co, (int)msg->data.result, error_str);
+#else
+            v = Py_BuildValue("(Ois)", (PyObject *)co, (int)msg->data.result, co->error);
+#endif
             /* Append curl object to list of objects which failed */
             if (v == NULL || PyList_Append(err_list, v) != 0) {
+                Py_XDECREF(error_str);
                 Py_XDECREF(v);
                 goto error;
             }
